"""Unit tests for Plone i18n

References:
http://i18n.kde.org/translation-howto/check-gui.html#check-msgfmt
"""

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
import I18NTestCase
import unittest,re
from testPoFiles import getPoFiles, Xprint

try:
    import commands
except ImportError:
    if os.name != 'posix':
        throw("The i18n tests only runs on posix systems, such as Linux, due to a dependency on Python's commands.getstatusoutput().")

class TestPOT(I18NTestCase.I18NTestCase):
    def testNoDuplicateMsgId(self):
        """Check that there are no duplicate msgid:s in plone.pot"""
        cmd='grep ^msgid ../plone.pot|sort|uniq --repeated'
        status = commands.getstatusoutput(cmd)
        assert len(status[1])  == 0, "Duplicate msgid:s were found:\n\n%s" % status[1]

class TestMsg(I18NTestCase.I18NTestCase):
    poFile = None

# This test isn't functional yet; msgfmt is pretty useless for automated testing until
# it can present the results in a simple machine-readable way. Now, the message varies
# in formatting depending on the results.
#
#    def testMsgFiles(self):
#        """Check that the .pot file is ok
#
#           "LC_ALL=C" is used so we don't get localised messages from msgfmt."""
#        status = commands.getstatusoutput('LC_ALL=C msgfmt --check --statistics ../plone.pot')
#        if status[0] != 0:
#            self.fail (status[1])

    def checkMsgExists(self,po,template):
        """Check that each existing message is translated and that there are no extra messages."""
        cmd='LC_ALL=C msgcmp --directory=.. %s %s' % (po,template)
        status = commands.getstatusoutput(cmd)
        if status[0] != 0:
            return status
        return None

    def testMsgExists(self):
        """
        """
        po = self.poFile
        poName = po.split('/')[-1]
        failed=[]
        if not po.endswith("plone-en.po"):
            res=self.checkMsgExists(po, 'plone.pot')
            if res!=None:
                output = res[1].split('\n')
                if len(output) > 10:
                    output = output[:10]
                    output.append('... <more errors>')
                output = '\n'.join(output)
                msg="Comparing '%s' with '%s' raised an error, exit code of msgcmp is: %s\n%s" % (poName, 'plone.pot' ,res[0], output)
                self.fail(msg)

tests=[]
for poFile in getPoFiles('..'):
    class TestOneMsg(TestMsg):
        poFile = poFile
    tests.append(TestOneMsg)

if __name__ == '__main__':
    framework()
else:
    import unittest
    def test_suite():
        suite = unittest.TestSuite()
        for test in tests:
            suite.addTest(unittest.makeSuite(test))
        return suite
